@@ -53,6 +53,7 @@
 
 #define MAX_NOTIFICATIONS 20
 
+#define WIDTH 400
 #define IMAGE_SIZE 48
 #define IDLE_SECONDS 30
 #define NOTIFICATION_BUS_NAME      "org.freedesktop.Notifications"
@@ -87,8 +88,8 @@ typedef struct {
 	guint id;
 	GtkWindow* nw;
 	Window src_window_xid;
-	guint has_timeout: 1;
-	guint paused: 1;
+	gboolean has_timeout;
+	gboolean paused;
 } NotifyTimeout;
 
 typedef struct {
@@ -130,12 +131,32 @@ static void monitor_notification_source_windows(NotifyDaemon* daemon, NotifyTime
 
 G_DEFINE_TYPE(NotifyDaemon, notify_daemon, G_TYPE_OBJECT);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+static void
+notify_daemon_get_preferred_width (GtkWidget *widget,
+                                   gint *min_width,
+                                   gint *nat_width)
+{
+        if (nat_width != NULL) {
+            *nat_width = WIDTH;
+        }
+}
+#endif
+
 static void notify_daemon_class_init(NotifyDaemonClass* daemon_class)
 {
 	GObjectClass* object_class = G_OBJECT_CLASS(daemon_class);
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+    GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (daemon_class);
+#endif
+
 	object_class->finalize = notify_daemon_finalize;
 
+#if GTK_CHECK_VERSION (3, 0, 0)
+    widget_class->get_preferred_width = notify_daemon_get_preferred_width;
+#endif
+
 	g_type_class_add_private(daemon_class, sizeof(NotifyDaemonPrivate));
 }
 
@@ -153,28 +174,26 @@ static void _notify_timeout_destroy(NotifyTimeout* nt)
 
 static gboolean do_exit(gpointer user_data)
 {
-	//g_debug("Exiting due to inactivity");
-	exit(1);
-
+	exit(0);
 	return FALSE;
 }
 
 static void add_exit_timeout(NotifyDaemon* daemon)
 {
+	g_assert (daemon != NULL);
+
 	if (daemon->priv->exit_timeout_source > 0)
-	{
 		return;
-	}
 
 	daemon->priv->exit_timeout_source = g_timeout_add_seconds(IDLE_SECONDS, do_exit, NULL);
 }
 
 static void remove_exit_timeout(NotifyDaemon* daemon)
 {
+	g_assert (daemon != NULL);
+
 	if (daemon->priv->exit_timeout_source == 0)
-	{
 		return;
-	}
 
 	g_source_remove(daemon->priv->exit_timeout_source);
 	daemon->priv->exit_timeout_source = 0;
