@@ -117,6 +117,7 @@ static void fill_background(GtkWidget* widget, WindowData* windata, cairo_t* cr)
 	GdkRGBA fg_color;
 	GdkRGBA bg_color;
 #else
+	GtkStyle *style;
 	GdkColor color;
 #endif
 	double r, g, b;
@@ -152,7 +153,9 @@ static void fill_background(GtkWidget* widget, WindowData* windata, cairo_t* cr)
 	cairo_stroke(cr);
 
 #else
-	color = widget->style->bg[GTK_STATE_NORMAL];
+	style = gtk_widget_get_style (widget);
+
+	color = style->bg[GTK_STATE_NORMAL];
 	r = (float) color.red / 65535.0;
 	g = (float) color.green / 65535.0;
 	b = (float) color.blue / 65535.0;
@@ -162,7 +165,7 @@ static void fill_background(GtkWidget* widget, WindowData* windata, cairo_t* cr)
 	/* Should we show urgency somehow?  Probably doesn't
 	 * have any meaningful value to the user... */
 
-	color = widget->style->text_aa[GTK_STATE_NORMAL];
+	color = style->text_aa[GTK_STATE_NORMAL];
 	r = (float) color.red / 65535.0;
 	g = (float) color.green / 65535.0;
 	b = (float) color.blue / 65535.0;
@@ -461,13 +464,13 @@ static void override_style(GtkWidget* widget, GtkStyle* previous_style)
 	GdkColor fg;
 	GdkColor bg;
 
-	style = gtk_style_copy(widget->style);
+	style = gtk_style_copy(gtk_widget_get_style (widget));
 
 	if (previous_style == NULL || (previous_style != NULL && (previous_style->bg[GTK_STATE_NORMAL].red != style->bg[GTK_STATE_NORMAL].red || previous_style->bg[GTK_STATE_NORMAL].green != style->bg[GTK_STATE_NORMAL].green || previous_style->bg[GTK_STATE_NORMAL].blue != style->bg[GTK_STATE_NORMAL].blue)))
 	{
 		state = (GtkStateType) 0;
 
-		while (state < (GtkStateType) G_N_ELEMENTS(widget->style->bg))
+		while (state < (GtkStateType) G_N_ELEMENTS(style->bg))
 		{
 			color_reverse(&style->bg[state], &bg);
 			gtk_widget_modify_bg(widget, state, &bg);
@@ -479,7 +482,7 @@ static void override_style(GtkWidget* widget, GtkStyle* previous_style)
 	{
 		state = (GtkStateType) 0;
 
-		while (state < (GtkStateType) G_N_ELEMENTS(widget->style->fg))
+		while (state < (GtkStateType) G_N_ELEMENTS(style->fg))
 		{
 			color_reverse(&style->fg[state], &fg);
 			gtk_widget_modify_fg(widget, state, &fg);
@@ -671,6 +674,9 @@ GtkWindow* create_notification(UrlClickedCb url_clicked)
 	gtk_box_pack_start(GTK_BOX(vbox), windata->summary_label, TRUE, TRUE, 0);
 	gtk_misc_set_alignment(GTK_MISC(windata->summary_label), 0, 0);
 	gtk_label_set_line_wrap(GTK_LABEL(windata->summary_label), TRUE);
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gtk_label_set_line_wrap_mode (GTK_LABEL (windata->summary_label), PANGO_WRAP_WORD_CHAR);
+#endif
 
 	atkobj = gtk_widget_get_accessible(windata->summary_label);
 	atk_object_set_description(atkobj, "Notification summary text.");
@@ -694,6 +700,9 @@ GtkWindow* create_notification(UrlClickedCb url_clicked)
 	gtk_box_pack_start(GTK_BOX(vbox), windata->body_label, TRUE, TRUE, 0);
 	gtk_misc_set_alignment(GTK_MISC(windata->body_label), 0, 0);
 	gtk_label_set_line_wrap(GTK_LABEL(windata->body_label), TRUE);
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gtk_label_set_line_wrap_mode (GTK_LABEL (windata->body_label), PANGO_WRAP_WORD_CHAR);
+#endif
 	g_signal_connect_swapped(G_OBJECT(windata->body_label), "activate-link", G_CALLBACK(windata->url_clicked), win);
 
 	atkobj = gtk_widget_get_accessible(windata->body_label);
@@ -899,7 +908,7 @@ static gboolean on_countdown_expose(GtkWidget* pie, GdkEventExpose* event, Windo
 	cairo_surface_t* surface;
 	double r, g, b;
 
-	context = gdk_cairo_create(gtk_widget_get_window(GDK_WINDOW(windata->pie_countdown)));
+	context = gdk_cairo_create(GDK_WINDOW(gtk_widget_get_window(windata->pie_countdown)));
 #if GTK_CHECK_VERSION(3, 0, 0)
 	style = gtk_widget_get_style_context(windata->win);
 #else
@@ -924,7 +933,7 @@ static gboolean on_countdown_expose(GtkWidget* pie, GdkEventExpose* event, Windo
 	g = color.green;
 	b = color.blue;
 #else
-	color = windata->win->style->bg[GTK_STATE_NORMAL];
+	color = style->bg[GTK_STATE_NORMAL];
 	r = (float) color.red / 65535.0;
 	g = (float) color.green / 65535.0;
 	b = (float) color.blue / 65535.0;
@@ -1063,7 +1072,7 @@ void clear_notification_actions(GtkWindow* nw)
 	windata->pie_countdown = NULL;
 
 	gtk_widget_hide(windata->actions_box);
-	gtk_container_foreach(GTK_CONTAINER(windata->actions_box), (GtkCallback) g_object_unref, NULL);
+	gtk_container_foreach(GTK_CONTAINER(windata->actions_box), (GtkCallback) gtk_widget_destroy, NULL);
 }
 
 void move_notification(GtkWidget* widget, int x, int y)
