@@ -441,7 +441,7 @@ static void draw_border(GtkWidget* widget, WindowData *windata, cairo_t* cr)
 
 	if (windata->has_arrow)
 	{
-		int i;
+		size_t i;
 
 		create_border_with_arrow(windata->win, windata);
 
@@ -456,7 +456,7 @@ static void draw_border(GtkWidget* widget, WindowData *windata, cairo_t* cr)
 #if GTK_CHECK_VERSION(3, 0, 0)
 		/* FIXME!!! */
 #else
-		gdk_window_shape_combine_region(windata->win->window, windata->window_region, 0, 0);
+		gdk_window_shape_combine_region (gtk_widget_get_window (windata->win), windata->window_region, 0, 0);
 #endif
 		g_free(windata->border_points);
 		windata->border_points = NULL;
@@ -718,6 +718,9 @@ GtkWindow* create_notification(UrlClickedCb url_clicked)
 	gtk_box_pack_start(GTK_BOX(hbox), windata->summary_label, TRUE, TRUE, 0);
 	gtk_misc_set_alignment(GTK_MISC(windata->summary_label), 0, 0);
 	gtk_label_set_line_wrap(GTK_LABEL(windata->summary_label), TRUE);
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gtk_label_set_line_wrap_mode (GTK_LABEL (windata->summary_label), PANGO_WRAP_WORD_CHAR);
+#endif
 
 	atkobj = gtk_widget_get_accessible(windata->summary_label);
 	atk_object_set_description(atkobj, "Notification summary text.");
@@ -734,7 +737,7 @@ GtkWindow* create_notification(UrlClickedCb url_clicked)
 	gtk_button_set_relief(GTK_BUTTON(close_button), GTK_RELIEF_NONE);
 	gtk_container_set_border_width(GTK_CONTAINER(close_button), 0);
 	//gtk_widget_set_size_request(close_button, 20, 20);
-	g_signal_connect_swapped(G_OBJECT(close_button), "clicked", G_CALLBACK(g_object_unref), win);
+	g_signal_connect_swapped(G_OBJECT(close_button), "clicked", G_CALLBACK(gtk_widget_destroy), win);
 
 	rcstyle = gtk_rc_style_new();
 	rcstyle->xthickness = rcstyle->ythickness = 0;
@@ -770,6 +773,9 @@ GtkWindow* create_notification(UrlClickedCb url_clicked)
 	gtk_box_pack_start(GTK_BOX(vbox), windata->body_label, TRUE, TRUE, 0);
 	gtk_misc_set_alignment(GTK_MISC(windata->body_label), 0, 0);
 	gtk_label_set_line_wrap(GTK_LABEL(windata->body_label), TRUE);
+#if GTK_CHECK_VERSION (3, 0, 0)
+	gtk_label_set_line_wrap_mode (GTK_LABEL (windata->body_label), PANGO_WRAP_WORD_CHAR);
+#endif
 	g_signal_connect(G_OBJECT(windata->body_label), "activate-link", G_CALLBACK(activate_link), windata);
 
 	atkobj = gtk_widget_get_accessible(windata->body_label);
@@ -925,7 +931,7 @@ static gboolean countdown_expose_cb(GtkWidget* pie, GdkEventExpose* event, Windo
 #endif
 
 	style = gtk_widget_get_style(windata->win);
-	context = gdk_cairo_create(GTK_WINDOW(gtk_widget_get_window(windata->pie_countdown)));
+	context = gdk_cairo_create(GDK_WINDOW(gtk_widget_get_window(windata->pie_countdown)));
 	GtkAllocation alloc;
 	gtk_widget_get_allocation(pie, &alloc);
 	cairo_set_operator(context, CAIRO_OPERATOR_SOURCE);
@@ -1047,7 +1053,7 @@ void clear_notification_actions(GtkWindow* nw)
 	windata->pie_countdown = NULL;
 
 	gtk_widget_hide(windata->actions_box);
-	gtk_container_foreach(GTK_CONTAINER(windata->actions_box), (GtkCallback) g_object_unref, NULL);
+	gtk_container_foreach(GTK_CONTAINER(windata->actions_box), (GtkCallback) gtk_widget_destroy, NULL);
 }
 
 void move_notification(GtkWidget* nw, int x, int y)
